@@ -1,3 +1,5 @@
+// SPDX-License-Identifier: Apache-2.0
+// 
 // Copyright 2008-2016 Conrad Sanderson (http://conradsanderson.id.au)
 // Copyright 2008-2016 National ICT Australia (NICTA)
 // 
@@ -35,6 +37,7 @@ template<typename eT, bool do_conj> class xtrans_mat;
 
 template<typename eT> class subview;
 template<typename eT> class subview_col;
+template<typename eT> class subview_cols;
 template<typename eT> class subview_row;
 template<typename eT> class subview_row_strans;
 template<typename eT> class subview_row_htrans;
@@ -67,6 +70,8 @@ template<typename eT>              class subview_cube_each1;
 template<typename eT, typename TB> class subview_cube_each2;
 template<typename eT, typename T1> class subview_cube_slices;
 
+template<typename eT, typename T1> class SpSubview_col_list;
+
 
 class SizeMat;
 class SizeCube;
@@ -78,8 +83,10 @@ class diskio;
 class op_strans;
 class op_htrans;
 class op_htrans2;
-class op_inv;
-class op_inv_sympd;
+class op_inv_gen_default;
+class op_inv_spd_default;
+class op_inv_gen_full;
+class op_inv_spd_full;
 class op_diagmat;
 class op_trimat;
 class op_vectorise_row;
@@ -144,9 +151,9 @@ struct traits_op_default
   template<typename T1>
   struct traits
     {
-    static const bool is_row  = false;
-    static const bool is_col  = false;
-    static const bool is_xvec = false;
+    static constexpr bool is_row  = false;
+    static constexpr bool is_col  = false;
+    static constexpr bool is_xvec = false;
     };
   };
 
@@ -156,9 +163,9 @@ struct traits_op_xvec
   template<typename T1>
   struct traits
     {
-    static const bool is_row  = false;
-    static const bool is_col  = false;
-    static const bool is_xvec = true;
+    static constexpr bool is_row  = false;
+    static constexpr bool is_col  = false;
+    static constexpr bool is_xvec = true;
     };
   };
 
@@ -168,9 +175,9 @@ struct traits_op_col
   template<typename T1>
   struct traits
     {
-    static const bool is_row  = false;
-    static const bool is_col  = true;
-    static const bool is_xvec = false;
+    static constexpr bool is_row  = false;
+    static constexpr bool is_col  = true;
+    static constexpr bool is_xvec = false;
     };
   };
 
@@ -180,9 +187,9 @@ struct traits_op_row
   template<typename T1>
   struct traits
     {
-    static const bool is_row  = true;
-    static const bool is_col  = false;
-    static const bool is_xvec = false;
+    static constexpr bool is_row  = true;
+    static constexpr bool is_col  = false;
+    static constexpr bool is_xvec = false;
     };
   };
 
@@ -192,9 +199,9 @@ struct traits_op_passthru
   template<typename T1>
   struct traits
     {
-    static const bool is_row  = T1::is_row;
-    static const bool is_col  = T1::is_col;
-    static const bool is_xvec = T1::is_xvec;
+    static constexpr bool is_row  = T1::is_row;
+    static constexpr bool is_col  = T1::is_col;
+    static constexpr bool is_xvec = T1::is_xvec;
     };
   };
 
@@ -204,9 +211,9 @@ struct traits_glue_default
   template<typename T1, typename T2>
   struct traits
     {
-    static const bool is_row  = false;
-    static const bool is_col  = false;
-    static const bool is_xvec = false;
+    static constexpr bool is_row  = false;
+    static constexpr bool is_col  = false;
+    static constexpr bool is_xvec = false;
     };
   };
 
@@ -216,9 +223,9 @@ struct traits_glue_or
   template<typename T1, typename T2>
   struct traits
     {
-    static const bool is_row  = (T1::is_row  || T2::is_row );
-    static const bool is_col  = (T1::is_col  || T2::is_col );
-    static const bool is_xvec = (T1::is_xvec || T2::is_xvec);
+    static constexpr bool is_row  = (T1::is_row  || T2::is_row );
+    static constexpr bool is_col  = (T1::is_col  || T2::is_col );
+    static constexpr bool is_xvec = (T1::is_xvec || T2::is_xvec);
     };
   };
 
@@ -253,12 +260,13 @@ template<                 typename T1, typename T2, typename eglue_type> class
 template<typename out_eT, typename T1, typename T2, typename  glue_type> class mtGlueCube;
 
 
-template<typename T1> class Proxy;
-template<typename T1> class ProxyCube;
+template<typename T1> struct Proxy;
+template<typename T1> struct ProxyCube;
 
 template<typename T1> class diagmat_proxy;
 
 template<typename T1> struct unwrap;
+template<typename T1> struct quasi_unwrap;
 template<typename T1> struct unwrap_cube;
 template<typename T1> struct unwrap_spmat;
 
@@ -269,7 +277,7 @@ struct state_type
   {
   #if   defined(ARMA_USE_OPENMP)
                 int  state;
-  #elif defined(ARMA_USE_CXX11_MUTEX)
+  #elif (!defined(ARMA_DONT_USE_STD_MUTEX))
     std::atomic<int> state;
   #else
                 int  state;
@@ -288,7 +296,7 @@ struct state_type
     #if   defined(ARMA_USE_OPENMP)
       #pragma omp atomic read
       out = state;
-    #elif defined(ARMA_USE_CXX11_MUTEX)
+    #elif (!defined(ARMA_DONT_USE_STD_MUTEX))
       out = state.load();
     #else
       out = state;
@@ -304,7 +312,7 @@ struct state_type
     #if   defined(ARMA_USE_OPENMP)
       #pragma omp atomic write
       state = in_state;
-    #elif defined(ARMA_USE_CXX11_MUTEX)
+    #elif (!defined(ARMA_DONT_USE_STD_MUTEX))
       state.store(in_state);
     #else
       state = in_state;
@@ -320,7 +328,7 @@ template<                 typename T1, typename T2, typename spglue_type> class
 template<typename out_eT, typename T1, typename T2, typename spglue_type> class mtSpGlue;
 
 
-template<typename T1> class SpProxy;
+template<typename T1> struct SpProxy;
 
 
 
@@ -329,6 +337,11 @@ struct arma_fixed_indicator   {};
 struct arma_reserve_indicator {};
 struct arma_layout_indicator  {};
 
+template<bool do_zeros> struct arma_initmode_indicator {};
+
+struct arma_zeros_indicator   : public arma_initmode_indicator<true > {};
+struct arma_nozeros_indicator : public arma_initmode_indicator<false> {};
+
 
 //! \addtogroup injector
 //! @{
@@ -347,7 +360,7 @@ static const injector_end_of_row<> endr = injector_end_of_row<>();
 //! @{
 
 
-enum file_type
+enum struct file_type : unsigned int
   {
   file_type_unknown,
   auto_detect,        //!< attempt to automatically detect the file type
@@ -359,11 +372,27 @@ enum file_type
   pgm_binary,         //!< Portable Grey Map (greyscale image)
   ppm_binary,         //!< Portable Pixel Map (colour image), used by the field and cube classes
   hdf5_binary,        //!< HDF5: open binary format, not specific to Armadillo, which can store arbitrary data
-  hdf5_binary_trans,  //!< [DO NOT USE - deprecated] as per hdf5_binary, but save/load the data with columns transposed to rows
-  coord_ascii         //!< simple co-ordinate format for sparse matrices (indices start at zero)
+  hdf5_binary_trans,  //!< [NOTE: DO NOT USE - deprecated] as per hdf5_binary, but save/load the data with columns transposed to rows
+  coord_ascii,        //!< simple co-ordinate format for sparse matrices (indices start at zero)
+  ssv_ascii,          //!< similar to csv_ascii; uses semicolon (;) instead of comma (,) as the separator
   };
 
 
+static constexpr file_type file_type_unknown  = file_type::file_type_unknown;
+static constexpr file_type auto_detect        = file_type::auto_detect;
+static constexpr file_type raw_ascii          = file_type::raw_ascii;
+static constexpr file_type arma_ascii         = file_type::arma_ascii;
+static constexpr file_type csv_ascii          = file_type::csv_ascii;
+static constexpr file_type raw_binary         = file_type::raw_binary;
+static constexpr file_type arma_binary        = file_type::arma_binary;
+static constexpr file_type pgm_binary         = file_type::pgm_binary;
+static constexpr file_type ppm_binary         = file_type::ppm_binary;
+static constexpr file_type hdf5_binary        = file_type::hdf5_binary;
+static constexpr file_type hdf5_binary_trans  = file_type::hdf5_binary_trans;
+static constexpr file_type coord_ascii        = file_type::coord_ascii;
+static constexpr file_type ssv_ascii          = file_type::ssv_ascii;
+
+
 struct hdf5_name;
 struct  csv_name;
 
@@ -372,33 +401,6 @@ struct  csv_name;
 
 
 
-//! \addtogroup fill
-//! @{
-
-namespace fill
-  {
-  struct fill_none  {};
-  struct fill_zeros {};
-  struct fill_ones  {};
-  struct fill_eye   {};
-  struct fill_randu {};
-  struct fill_randn {};
-  
-  template<typename fill_type> 
-  struct fill_class { inline fill_class() {} };
-  
-  static const fill_class<fill_none > none;
-  static const fill_class<fill_zeros> zeros;
-  static const fill_class<fill_ones > ones;
-  static const fill_class<fill_eye  > eye;
-  static const fill_class<fill_randu> randu;
-  static const fill_class<fill_randn> randn;
-  }
-
-//! @}
-
-
-
 //! \addtogroup fn_spsolve
 //! @{
 
@@ -444,3 +446,26 @@ struct superlu_opts : public spsolve_opts_base
 
 
 //! @}
+
+
+
+//! \ingroup fn_eigs_sym fs_eigs_gen
+//! @{
+
+
+struct eigs_opts
+  {
+  double       tol;     // tolerance
+  unsigned int maxiter; // max iterations
+  unsigned int subdim;  // subspace dimension
+  
+  inline eigs_opts()
+    {
+    tol     = 0.0;
+    maxiter = 1000;
+    subdim  = 0;
+    }
+  };
+
+
+//! @}
