@@ -28,53 +28,7 @@ arrayops::copy(eT* dest, const eT* src, const uword n_elem)
   {
   if( (dest == src) || (n_elem == 0) )  { return; }
   
-  if(is_cx<eT>::no)
-    {
-    if(n_elem <= 9)
-      {
-      arrayops::copy_small(dest, src, n_elem);
-      }
-    else
-      {
-      std::memcpy(dest, src, n_elem*sizeof(eT));
-      }
-    }
-  else
-    {
-    std::memcpy(dest, src, n_elem*sizeof(eT));
-    }
-  }
-
-
-
-template<typename eT>
-arma_cold
-inline
-void
-arrayops::copy_small(eT* dest, const eT* src, const uword n_elem)
-  {
-  switch(n_elem)
-    {
-    case  9:  dest[ 8] = src[ 8];
-    // fallthrough
-    case  8:  dest[ 7] = src[ 7];
-    // fallthrough
-    case  7:  dest[ 6] = src[ 6];
-    // fallthrough
-    case  6:  dest[ 5] = src[ 5];
-    // fallthrough
-    case  5:  dest[ 4] = src[ 4];
-    // fallthrough
-    case  4:  dest[ 3] = src[ 3];
-    // fallthrough
-    case  3:  dest[ 2] = src[ 2];
-    // fallthrough
-    case  2:  dest[ 1] = src[ 1];
-    // fallthrough
-    case  1:  dest[ 0] = src[ 0];
-    // fallthrough
-    default:  ;
-    }
+  std::memcpy(dest, src, n_elem*sizeof(eT));
   }
 
 
@@ -679,14 +633,7 @@ arrayops::inplace_set(eT* dest, const eT val, const uword n_elem)
     }
   else
     {
-    if( (n_elem <= 9) && (is_cx<eT>::no) )
-      {
-      arrayops::inplace_set_small(dest, val, n_elem);
-      }
-    else
-      {
-      arrayops::inplace_set_simple(dest, val, n_elem);
-      }
+    arrayops::inplace_set_simple(dest, val, n_elem);
     }
   }
 
@@ -745,38 +692,6 @@ arrayops::inplace_set_base(eT* dest, const eT val, const uword n_elem)
 
 
 
-template<typename eT>
-arma_cold
-inline
-void
-arrayops::inplace_set_small(eT* dest, const eT val, const uword n_elem)
-  {
-  switch(n_elem)
-    {
-    case  9: dest[ 8] = val;
-    // fallthrough
-    case  8: dest[ 7] = val;
-    // fallthrough
-    case  7: dest[ 6] = val;
-    // fallthrough
-    case  6: dest[ 5] = val;
-    // fallthrough
-    case  5: dest[ 4] = val;
-    // fallthrough
-    case  4: dest[ 3] = val;
-    // fallthrough
-    case  3: dest[ 2] = val;
-    // fallthrough
-    case  2: dest[ 1] = val;
-    // fallthrough
-    case  1: dest[ 0] = val;
-    // fallthrough
-    default:;
-    }
-  }
-
-
-
 template<typename eT, const uword n_elem>
 arma_hot
 inline
@@ -1156,18 +1071,13 @@ arrayops::is_finite(const eT* src, const uword n_elem)
     const eT val_i = (*src);  src++;
     const eT val_j = (*src);  src++;
     
-    if( (arma_isfinite(val_i) == false) || (arma_isfinite(val_j) == false) )
-      {
-      return false;
-      }
+    if(arma_isfinite(val_i) == false)  { return false; }
+    if(arma_isfinite(val_j) == false)  { return false; }
     }
   
   if((j-1) < n_elem)
     {
-    if(arma_isfinite(*src) == false)
-      {
-      return false;
-      }
+    if(arma_isfinite(*src) == false)  { return false; }
     }
   
   return true;
