@@ -25,11 +25,11 @@ template<typename T1>
 arma_warn_unused
 inline
 typename enable_if2< is_arma_type<T1>::value, const Op<T1, op_reshape> >::result
-reshape(const T1& X, const uword in_n_rows, const uword in_n_cols)
+reshape(const T1& X, const uword new_n_rows, const uword new_n_cols)
   {
   arma_extra_debug_sigprint();
   
-  return Op<T1, op_reshape>(X, in_n_rows, in_n_cols);
+  return Op<T1, op_reshape>(X, new_n_rows, new_n_cols);
   }
 
 
@@ -52,7 +52,7 @@ template<typename T1>
 arma_deprecated
 inline
 const Op<T1, op_reshape_old>
-reshape(const Base<typename T1::elem_type,T1>& X, const uword in_n_rows, const uword in_n_cols, const uword dim)  //!< NOTE: don't use this form: it will be removed
+reshape(const Base<typename T1::elem_type,T1>& X, const uword new_n_rows, const uword new_n_cols, const uword dim)  //!< NOTE: don't use this form: it will be removed
   {
   arma_extra_debug_sigprint();
   
@@ -60,7 +60,7 @@ reshape(const Base<typename T1::elem_type,T1>& X, const uword in_n_rows, const u
   
   arma_debug_check( (dim > 1), "reshape(): parameter 'dim' must be 0 or 1" );
   
-  return Op<T1, op_reshape_old>(X.get_ref(), in_n_rows, in_n_cols, dim, 'j');
+  return Op<T1, op_reshape_old>(X.get_ref(), new_n_rows, new_n_cols, dim, 'j');
   }
 
 
@@ -69,11 +69,11 @@ template<typename T1>
 arma_warn_unused
 inline
 const OpCube<T1, op_reshape>
-reshape(const BaseCube<typename T1::elem_type,T1>& X, const uword in_n_rows, const uword in_n_cols, const uword in_n_slices)
+reshape(const BaseCube<typename T1::elem_type,T1>& X, const uword new_n_rows, const uword new_n_cols, const uword new_n_slices)
   {
   arma_extra_debug_sigprint();
   
-  return OpCube<T1, op_reshape>(X.get_ref(), in_n_rows, in_n_cols, in_n_slices);
+  return OpCube<T1, op_reshape>(X.get_ref(), new_n_rows, new_n_cols, new_n_slices);
   }
 
 
@@ -95,11 +95,11 @@ template<typename T1>
 arma_warn_unused
 inline
 const SpOp<T1, spop_reshape>
-reshape(const SpBase<typename T1::elem_type, T1>& X, const uword in_n_rows, const uword in_n_cols)
+reshape(const SpBase<typename T1::elem_type, T1>& X, const uword new_n_rows, const uword new_n_cols)
   {
   arma_extra_debug_sigprint();
   
-  return SpOp<T1, spop_reshape>(X.get_ref(), in_n_rows, in_n_cols);
+  return SpOp<T1, spop_reshape>(X.get_ref(), new_n_rows, new_n_cols);
   }
 
 
