@@ -1,3 +1,5 @@
+// SPDX-License-Identifier: Apache-2.0
+// 
 // Copyright 2008-2016 Conrad Sanderson (http://conradsanderson.id.au)
 // Copyright 2008-2016 National ICT Australia (NICTA)
 // 
@@ -71,6 +73,12 @@ sp_auxlib::eigs_sym(Col<eT>& eigval, Mat<eT>& eigvec, const SpBase<eT, T1>& X, c
     if(is_cx<eT>::yes)  { arma_debug_warn_level(1, "eigs_sym(): given matrix is not hermitian"); }
     }
   
+  if(arma_config::check_nonfinite && U.M.has_nonfinite())
+    {
+    arma_debug_warn_level(3, "eigs_sym(): detected non-finite elements");
+    return false;
+    }
+  
   // TODO: investigate optional redirection of "sm" to ARPACK as it's capable of shift-invert;
   // TODO: in shift-invert mode, "sm" maps to "lm" of the shift-inverted matrix (with sigma = 0)
   
@@ -118,6 +126,12 @@ sp_auxlib::eigs_sym(Col<eT>& eigval, Mat<eT>& eigvec, const SpBase<eT, T1>& X, c
     if(is_cx<eT>::yes)  { arma_debug_warn_level(1, "eigs_sym(): given matrix is not hermitian"); }
     }
   
+  if(arma_config::check_nonfinite && U.M.has_nonfinite())
+    {
+    arma_debug_warn_level(3, "eigs_sym(): detected non-finite elements");
+    return false;
+    }
+  
   #if   (defined(ARMA_USE_NEWARP) && defined(ARMA_USE_SUPERLU))
     {
     return sp_auxlib::eigs_sym_newarp(eigval, eigvec, U.M, n_eigvals, sigma, opts);
@@ -480,6 +494,8 @@ sp_auxlib::eigs_sym_arpack(Col<eT>& eigval, Mat<eT>& eigvec, const SpMat<eT>& X,
     podarray<blas_int> select(ncv); // Logical array of dimension NCV.
     blas_int ldz = n;
     
+    select.zeros();
+    
     // seupd() will output directly into the eigval and eigvec objects.
     eigval.zeros(n_eigvals);
     eigvec.zeros(n, n_eigvals);
@@ -520,6 +536,12 @@ sp_auxlib::eigs_gen(Col< std::complex<T> >& eigval, Mat< std::complex<T> >& eigv
   
   arma_debug_check( (U.M.is_square() == false), "eigs_gen(): given matrix must be square sized" );
   
+  if(arma_config::check_nonfinite && U.M.has_nonfinite())
+    {
+    arma_debug_warn_level(3, "eigs_gen(): detected non-finite elements");
+    return false;
+    }
+  
   // TODO: investigate optional redirection of "sm" to ARPACK as it's capable of shift-invert;
   // TODO: in shift-invert mode, "sm" maps to "lm" of the shift-inverted matrix (with sigma = 0)
   
@@ -561,6 +583,12 @@ sp_auxlib::eigs_gen(Col< std::complex<T> >& eigval, Mat< std::complex<T> >& eigv
   
   arma_debug_check( (U.M.is_square() == false), "eigs_gen(): given matrix must be square sized" );
   
+  if(arma_config::check_nonfinite && U.M.has_nonfinite())
+    {
+    arma_debug_warn_level(3, "eigs_gen(): detected non-finite elements");
+    return false;
+    }
+  
   #if (defined(ARMA_USE_ARPACK) && defined(ARMA_USE_SUPERLU))
     {
     constexpr form_type form_val = form_sigma;
@@ -841,7 +869,7 @@ sp_auxlib::eigs_gen_arpack(Col< std::complex<T> >& eigval, Mat< std::complex<T>
       }
     
     if(info != 0)  { return false; }
-
+    
     // The process has converged, and now we need to recover the actual eigenvectors using neupd().
     blas_int rvec = 1; // .TRUE
     blas_int nev  = blas_int(n_eigvals);
@@ -856,9 +884,11 @@ sp_auxlib::eigs_gen_arpack(Col< std::complex<T> >& eigval, Mat< std::complex<T>
     blas_int ldz = n;
     podarray<T>        workev(3 * ncv);
     
+    select.zeros();
     dr.zeros();
     di.zeros();
     z.zeros();
+    workev.zeros();
     
     arpack::neupd(&rvec, &howmny, select.memptr(), dr.memptr(), di.memptr(), z.memptr(), &ldz, (T*) &sigmar, (T*) &sigmai, workev.memptr(), &bmat, &n, which, &nev, &tol, resid.memptr(), &ncv, v.memptr(), &ldv, iparam.memptr(), ipntr.memptr(), workd.memptr(), workl.memptr(), &lworkl, rwork.memptr(), &info);
     
@@ -938,6 +968,12 @@ sp_auxlib::eigs_gen(Col< std::complex<T> >& eigval, Mat< std::complex<T> >& eigv
   
   arma_debug_check( (U.M.is_square() == false), "eigs_gen(): given matrix must be square sized" );
   
+  if(arma_config::check_nonfinite && U.M.has_nonfinite())
+    {
+    arma_debug_warn_level(3, "eigs_gen(): detected non-finite elements");
+    return false;
+    }
+  
   constexpr std::complex<T> sigma = T(0);
   
   return sp_auxlib::eigs_gen<T, false>(eigval, eigvec, U.M, n_eigvals, form_val, sigma, opts);
@@ -957,6 +993,12 @@ sp_auxlib::eigs_gen(Col< std::complex<T> >& eigval, Mat< std::complex<T> >& eigv
   
   arma_debug_check( (U.M.is_square() == false), "eigs_gen(): given matrix must be square sized" );
   
+  if(arma_config::check_nonfinite && U.M.has_nonfinite())
+    {
+    arma_debug_warn_level(3, "eigs_gen(): detected non-finite elements");
+    return false;
+    }
+  
   #if (defined(ARMA_USE_ARPACK) && defined(ARMA_USE_SUPERLU))
     {
     constexpr form_type form_val = form_sigma;
@@ -1088,6 +1130,11 @@ sp_auxlib::eigs_gen(Col< std::complex<T> >& eigval, Mat< std::complex<T> >& eigv
     blas_int ldz = n;
     podarray<std::complex<T>> workev(2 * ncv);
     
+    select.zeros();
+    d.zeros();
+    z.zeros();
+    workev.zeros();
+    
     // Prepare the outputs; neupd() will write directly to them.
     eigval.zeros(n_eigvals);
     eigvec.zeros(n, n_eigvals);
@@ -1161,6 +1208,12 @@ sp_auxlib::spsolve_simple(Mat<typename T1::elem_type>& X, const SpBase<typename
     
     if(A.n_nonzero == uword(0))  { X.soft_reset(); return false; }
     
+    if(arma_config::check_nonfinite && (A.has_nonfinite() || X.has_nonfinite()))
+      {
+      arma_debug_warn_level(3, "spsolve(): detected non-finite elements");
+      return false;
+      }
+    
     if(arma_config::debug)
       {
       bool overflow = false;
@@ -1283,13 +1336,16 @@ sp_auxlib::spsolve_refine(Mat<typename T1::elem_type>& X, typename T1::pod_type&
     
     X.zeros(A.n_cols, B.n_cols);  // set the elements to zero, as we don't trust the SuperLU spaghetti code
     
-    if(A.is_empty() || B.is_empty())
-      {
-      return true;
-      }
+    if(A.is_empty() || B.is_empty())  { return true; }
     
     if(A.n_nonzero == uword(0))  { X.soft_reset(); return false; }
     
+    if(arma_config::check_nonfinite && (A.has_nonfinite() || X.has_nonfinite()))
+      {
+      arma_debug_warn_level(3, "spsolve(): detected non-finite elements");
+      return false;
+      }
+    
     if(arma_config::debug)
       {
       bool overflow;
@@ -1341,13 +1397,13 @@ sp_auxlib::spsolve_refine(Mat<typename T1::elem_type>& X, typename T1::pod_type&
     
     superlu_stat_wrangler stat;
     
-    char equed[8];       // extra characters for paranoia
-    T    rpg   = T(0);
-    T    rcond = T(0);
-    int  info  = int(0); // Return code.
+    char equed[8] = {};     // extra characters for paranoia
+    T    rpg      = T(0);
+    T    rcond    = T(0);
+    int  info     = int(0); // Return code.
     
-    char  work[8];
-    int  lwork = int(0);  // 0 means superlu will allocate memory
+    char  work[8] = {};
+    int  lwork    = int(0);  // 0 means superlu will allocate memory
     
     arma_extra_debug_print("superlu::gssvx()");
     superlu::gssvx<eT>(&options, a.get_ptr(), perm_c.get_ptr(), perm_r.get_ptr(), etree.get_ptr(), equed, R.get_ptr(), C.get_ptr(), l.get_ptr(), u.get_ptr(), &work[0], lwork, b.get_ptr(), x.get_ptr(), &rpg, &rcond, ferr.get_ptr(), berr.get_ptr(), &glu, &mu, stat.get_ptr(), &info);
@@ -1374,7 +1430,7 @@ sp_auxlib::spsolve_refine(Mat<typename T1::elem_type>& X, typename T1::pod_type&
     else
     if(info > int(A.n_cols+1))
       {
-      arma_debug_warn_level(1, "spsolve(): memory allocation failure: could not allocate ", (info - int(A.n_cols)), " bytes");
+      arma_debug_warn_level(1, "spsolve(): memory allocation failure");
       }
     else
     if(info < 0)
@@ -1444,15 +1500,14 @@ sp_auxlib::spsolve_refine(Mat<typename T1::elem_type>& X, typename T1::pod_type&
 //     
 //     superlu_stat_wrangler stat;
 //     
-//     int   panel_size = superlu::sp_ispec_environ(1);
-//     int   relax      = superlu::sp_ispec_environ(2);
-//     float drop_tol   = 0.0;
-//     int   slu_info   = 0; // Return code.
+//     int panel_size = superlu::sp_ispec_environ(1);
+//     int relax      = superlu::sp_ispec_environ(2);
+//     int slu_info   = 0; // Return code.
 //     
 //     arma_extra_debug_print("superlu::gstrf()");
 //     superlu::get_permutation_c(options.ColPerm, a.get_ptr(), perm_c.get_ptr());
 //     superlu::sp_preorder_mat(&options, a.get_ptr(), perm_c.get_ptr(), etree.get_ptr(), aC.get_ptr());
-//     superlu::gstrf<T>(&options, aC.get_ptr(), drop_tol, relax, panel_size, etree.get_ptr(), NULL, lwork, perm_c.get_ptr(), perm_r.get_ptr(), l.get_ptr(), u.get_ptr(), &Glu, stat.get_ptr(), &slu_info);
+//     superlu::gstrf<T>(&options, aC.get_ptr(), relax, panel_size, etree.get_ptr(), NULL, lwork, perm_c.get_ptr(), perm_r.get_ptr(), l.get_ptr(), u.get_ptr(), &Glu, stat.get_ptr(), &slu_info);
 //     
 //     if(slu_info != 0)  { return T(0); }
 //     
@@ -1951,7 +2006,7 @@ sp_auxlib::run_aupd_plain
     n = X.n_rows; // The size of the matrix (should already be set outside).
     blas_int nev = n_eigvals;
     
-    resid.set_size(n);
+    resid.zeros(n);
     
     // Two contraints on NCV: (NCV > NEV) for sym problems or
     // (NCV > NEV + 2) for gen problems and (NCV <= N)
@@ -1964,8 +2019,8 @@ sp_auxlib::run_aupd_plain
     if(ncv < (nev + (sym ? 1 : 3))) { ncv = (nev + (sym ? 1 : 3)); }
     if(ncv > n                    ) { ncv = n;                     }
     
-    v.set_size(n * ncv); // Array N by NCV (output).
-    rwork.set_size(ncv); // Work array of size NCV for complex calls.
+    v.zeros(n * ncv); // Array N by NCV (output).
+    rwork.zeros(ncv); // Work array of size NCV for complex calls.
     ldv = n; // "Leading dimension of V exactly as declared in the calling program."
     
     // IPARAM: integer array of length 11.
@@ -1975,16 +2030,16 @@ sp_auxlib::run_aupd_plain
     iparam(6) = 1; // Mode 1: A * x = lambda * x.
     
     // IPNTR: integer array of length 14 (output).
-    ipntr.set_size(14);
+    ipntr.zeros(14);
     
     // Real work array used in the basic Arnoldi iteration for reverse communication.
-    workd.set_size(3 * n);
+    workd.zeros(3 * n);
     
     // lworkl must be at least 3 * NCV^2 + 6 * NCV.
     lworkl = 3 * (ncv * ncv) + 6 * ncv;
     
     // Real work array of length lworkl.
-    workl.set_size(lworkl);
+    workl.zeros(lworkl);
     
     info = 0; // Set to 0 initially to use random initial vector.
     
@@ -2076,6 +2131,7 @@ sp_auxlib::run_aupd_plain
     arma_ignore(sym);
     arma_ignore(n);
     arma_ignore(tol);
+    arma_ignore(maxiter);
     arma_ignore(resid);
     arma_ignore(ncv);
     arma_ignore(v);
@@ -2122,7 +2178,7 @@ sp_auxlib::run_aupd_shiftinvert
     n = X.n_rows; // The size of the matrix (should already be set outside).
     blas_int nev = n_eigvals;
     
-    resid.set_size(n);
+    resid.zeros(n);
     
     // Two contraints on NCV: (NCV > NEV) for sym problems or
     // (NCV > NEV + 2) for gen problems and (NCV <= N)
@@ -2135,8 +2191,8 @@ sp_auxlib::run_aupd_shiftinvert
     if(ncv < (nev + (sym ? 1 : 3))) { ncv = (nev + (sym ? 1 : 3)); }
     if(ncv > n                    ) { ncv = n;                     }
     
-    v.set_size(n * ncv); // Array N by NCV (output).
-    rwork.set_size(ncv); // Work array of size NCV for complex calls.
+    v.zeros(n * ncv); // Array N by NCV (output).
+    rwork.zeros(ncv); // Work array of size NCV for complex calls.
     ldv = n; // "Leading dimension of V exactly as declared in the calling program."
     
     // IPARAM: integer array of length 11.
@@ -2149,16 +2205,16 @@ sp_auxlib::run_aupd_shiftinvert
     iparam(6) = 3; // Mode 3:  A * x = lambda * M * x, M symmetric semi-definite. OP = inv[A - sigma*M]*M  (A complex)  or  Real_Part{ inv[A - sigma*M]*M }  (A real)  and  B = M.
     
     // IPNTR: integer array of length 14 (output).
-    ipntr.set_size(14);
+    ipntr.zeros(14);
     
     // Real work array used in the basic Arnoldi iteration for reverse communication.
-    workd.set_size(3 * n);
+    workd.zeros(3 * n);
     
     // lworkl must be at least 3 * NCV^2 + 6 * NCV.
     lworkl = 3 * (ncv * ncv) + 6 * ncv;
     
     // Real work array of length lworkl.
-    workl.set_size(lworkl);
+    workl.zeros(lworkl);
     
     info = 0; // Set to 0 initially to use random initial vector.
     
@@ -2216,15 +2272,14 @@ sp_auxlib::run_aupd_shiftinvert
     
     superlu_stat_wrangler stat;
     
-    int   panel_size = superlu::sp_ispec_environ(1);
-    int   relax      = superlu::sp_ispec_environ(2);
-    float drop_tol   = 0.0;
-    int   slu_info   = 0; // Return code.
+    int panel_size = superlu::sp_ispec_environ(1);
+    int relax      = superlu::sp_ispec_environ(2);
+    int slu_info   = 0; // Return code.
     
     arma_extra_debug_print("superlu::gstrf()");
     superlu::get_permutation_c(options.ColPerm, x.get_ptr(), perm_c.get_ptr());
     superlu::sp_preorder_mat(&options, x.get_ptr(), perm_c.get_ptr(), etree.get_ptr(), xC.get_ptr());
-    superlu::gstrf<T>(&options, xC.get_ptr(), drop_tol, relax, panel_size, etree.get_ptr(), NULL, lwork, perm_c.get_ptr(), perm_r.get_ptr(), l.get_ptr(), u.get_ptr(), &Glu, stat.get_ptr(), &slu_info);
+    superlu::gstrf<T>(&options, xC.get_ptr(), relax, panel_size, etree.get_ptr(), NULL, lwork, perm_c.get_ptr(), perm_r.get_ptr(), l.get_ptr(), u.get_ptr(), &Glu, stat.get_ptr(), &slu_info);
     
     if(slu_info != 0)
       {
@@ -2331,6 +2386,7 @@ sp_auxlib::run_aupd_shiftinvert
     arma_ignore(sym);
     arma_ignore(n);
     arma_ignore(tol);
+    arma_ignore(maxiter);
     arma_ignore(resid);
     arma_ignore(ncv);
     arma_ignore(v);
